@@ -3,6 +3,8 @@ try:
 except ImportError:
     import json  # flake8: noqa
 import requests
+import hashlib
+import hmac
 
 try:
     from urllib.parse import urlencode
@@ -10,11 +12,13 @@ except ImportError:
     from urllib import urlencode
 from decimal import Decimal
 
+import six
+
 from facepy.exceptions import *
 
 
 class GraphAPI(object):
-    def __init__(self, oauth_token=False, url='https://graph.facebook.com', verify_ssl_certificate=True):
+    def __init__(self, oauth_token=False, url='https://graph.facebook.com', verify_ssl_certificate=True, appsecret=False):
         """
         Initialize GraphAPI with an OAuth access token.
 
@@ -24,6 +28,7 @@ class GraphAPI(object):
         self.session = requests.session()
         self.url = url.strip('/')
         self.verify_ssl_certificate = verify_ssl_certificate
+        self.appsecret = appsecret
 
     @classmethod
     def for_application(self, id, secret_key):
@@ -150,9 +155,27 @@ class GraphAPI(object):
             if 'body' in request:
                 request['body'] = urlencode(request['body'])
 
-        responses = self.post(
-            batch=json.dumps(requests)
-        )
+        def _grouper(complete_list, n=1):
+            """
+            Batches a list into constant size chunks.
+
+            :param complete_list: A input list (not a generator).
+            :param n: The size of the chunk.
+
+            Adapted from <http://stackoverflow.com/questions/312443/how-do-you-split-a-list-into-evenly-sized-chunks-in-python>
+            """
+
+            for i in range(0, len(complete_list), n):
+                yield complete_list[i:i + n]
+
+        responses = []
+
+        # Maximum batch size for Facebook is 50 so split up requests
+        # https://developers.facebook.com/docs/graph-api/making-multiple-requests/#limits
+        for group in _grouper(requests, 50):
+            responses += self.post(
+                batch=json.dumps(group)
+            )
 
         for response, request in zip(responses, requests):
 
@@ -197,8 +220,12 @@ class GraphAPI(object):
         :param page: A boolean describing whether to return an iterator that iterates over each page of results.
         :param retry: An integer describing how many times the request may be retried.
         """
-        
+
         if(data):
+            data = dict(
+                 (k.replace('_sqbro_', '['), v) for k, v in data.items())
+            data = dict(
+                 (k.replace('_sqbrc_', ']'), v) for k, v in data.items())
             data = dict(
                  (k.replace('__', ':'), v) for k, v in data.items())
         data = data or {}
@@ -225,7 +252,7 @@ class GraphAPI(object):
                     for key in files:
                         data.pop(key)
 
-                    response = self.session.request(method, url, data=data, files=files, 
+                    response = self.session.request(method, url, data=data, files=files,
                         verify=self.verify_ssl_certificate)
             except requests.RequestException as exception:
                 raise HTTPError(exception)
@@ -252,18 +279,24 @@ class GraphAPI(object):
 
         # Convert option lists to comma-separated values.
         for key in data:
-            if isinstance(data[key], (list, set, tuple)) and all([isinstance(item, str) for item in data[key]]):
+            if isinstance(data[key], (list, set, tuple)) and all([isinstance(item, six.string_types) for item in data[key]]):
                 data[key] = ','.join(data[key])
 
         # Support absolute paths too
         if not path.startswith('/'):
-            path = '/' + str(path)
+            if six.PY2:
+                path = '/' + six.text_type(path.decode('utf-8'))
+            else:
+                path = '/' + path
 
         url = '%s%s' % (self.url, path)
 
         if self.oauth_token:
             data['access_token'] = self.oauth_token
 
+        if self.appsecret and self.oauth_token:
+            data['appsecret_proof'] = self._generate_appsecret_proof()
+
         try:
             if page:
                 return paginate(method, url, data)
@@ -325,5 +358,19 @@ class GraphAPI(object):
 
         return data
 
+    def _generate_appsecret_proof(self):
+        """
+        Returns a SHA256 of the oauth_token signed by appsecret.
+        https://developers.facebook.com/docs/graph-api/securing-requests/
+        """
+        if six.PY2:
+            key = self.appsecret
+            message = self.oauth_token
+        else:
+            key = bytes(self.appsecret, 'utf-8')
+            message = bytes(self.oauth_token, 'utf-8')
+
+        return hmac.new(key, message, hashlib.sha256).hexdigest()
+
     # Proxy exceptions for ease of use and backwards compatibility.
     FacebookError, OAuthError, HTTPError = FacebookError, OAuthError, HTTPError
